# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .LotteryModel import LotteryModel
from .options.LotteryModelOptions import LotteryModelOptions
from .enum.LotteryModelMode import LotteryModelMode
from .enum.LotteryModelMethod import LotteryModelMethod
from .options.LotteryModelMethodIsPrizeTableOptions import LotteryModelMethodIsPrizeTableOptions
from .options.LotteryModelMethodIsScriptOptions import LotteryModelMethodIsScriptOptions
from .PrizeTable import PrizeTable
from .options.PrizeTableOptions import PrizeTableOptions
from .Prize import Prize
from .options.PrizeOptions import PrizeOptions
from .enum.PrizeType import PrizeType
from .options.PrizeTypeIsActionOptions import PrizeTypeIsActionOptions
from .options.PrizeTypeIsPrizeTableOptions import PrizeTypeIsPrizeTableOptions
from .PrizeLimit import PrizeLimit
from .options.PrizeLimitOptions import PrizeLimitOptions
from .DrawnPrize import DrawnPrize
from .options.DrawnPrizeOptions import DrawnPrizeOptions
from .BoxItem import BoxItem
from .options.BoxItemOptions import BoxItemOptions
from .CurrentMasterData import CurrentMasterData


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .CounterScopeModel import CounterScopeModel
from .options.CounterScopeModelOptions import CounterScopeModelOptions
from .enum.CounterScopeModelResetType import CounterScopeModelResetType
from .enum.CounterScopeModelResetDayOfWeek import CounterScopeModelResetDayOfWeek
from .options.CounterScopeModelResetTypeIsNotResetOptions import CounterScopeModelResetTypeIsNotResetOptions
from .options.CounterScopeModelResetTypeIsDailyOptions import CounterScopeModelResetTypeIsDailyOptions
from .options.CounterScopeModelResetTypeIsWeeklyOptions import CounterScopeModelResetTypeIsWeeklyOptions
from .options.CounterScopeModelResetTypeIsMonthlyOptions import CounterScopeModelResetTypeIsMonthlyOptions
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .CounterModel import CounterModel
from .options.CounterModelOptions import CounterModelOptions
from .MissionGroupModel import MissionGroupModel
from .options.MissionGroupModelOptions import MissionGroupModelOptions
from .enum.MissionGroupModelResetType import MissionGroupModelResetType
from .enum.MissionGroupModelResetDayOfWeek import MissionGroupModelResetDayOfWeek
from .options.MissionGroupModelResetTypeIsNotResetOptions import MissionGroupModelResetTypeIsNotResetOptions
from .options.MissionGroupModelResetTypeIsDailyOptions import MissionGroupModelResetTypeIsDailyOptions
from .options.MissionGroupModelResetTypeIsWeeklyOptions import MissionGroupModelResetTypeIsWeeklyOptions
from .options.MissionGroupModelResetTypeIsMonthlyOptions import MissionGroupModelResetTypeIsMonthlyOptions
from .MissionTaskModel import MissionTaskModel
from .options.MissionTaskModelOptions import MissionTaskModelOptions
from .enum.MissionTaskModelTargetResetType import MissionTaskModelTargetResetType
from .ScopedValue import ScopedValue
from .options.ScopedValueOptions import ScopedValueOptions
from .enum.ScopedValueResetType import ScopedValueResetType
from .CurrentMasterData import CurrentMasterData


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .Scope import Scope
from .options.CategoryModelOptions import CategoryModelOptions
from .options.CategoryModelScopeIsGlobalOptions import CategoryModelScopeIsGlobalOptions
from .options.CategoryModelScopeIsScopedOptions import CategoryModelScopeIsScopedOptions
from .enum.CategoryModelOrderDirection import CategoryModelOrderDirection
from .enum.CategoryModelScope import CategoryModelScope


class CategoryModel:
    name: str
    order_direction: CategoryModelOrderDirection
    scope: CategoryModelScope
    unique_by_user_id: bool
    metadata: Optional[str] = None
    minimum_value: Optional[int] = None
    maximum_value: Optional[int] = None
    sum: Optional[bool] = None
    calculate_fixed_timing_hour: Optional[int] = None
    calculate_fixed_timing_minute: Optional[int] = None
    calculate_interval_minutes: Optional[int] = None
    additional_scopes: Optional[List[Scope]] = None
    entry_period_event_id: Optional[str] = None
    access_period_event_id: Optional[str] = None
    ignore_user_ids: Optional[List[str]] = None
    generation: Optional[str] = None

    def __init__(
        self,
        name: str,
        order_direction: CategoryModelOrderDirection,
        scope: CategoryModelScope,
        unique_by_user_id: bool,
        options: Optional[CategoryModelOptions] = CategoryModelOptions(),
    ):
        self.name = name
        self.order_direction = order_direction
        self.scope = scope
        self.unique_by_user_id = unique_by_user_id
        self.metadata = options.metadata if options.metadata else None
        self.minimum_value = options.minimum_value if options.minimum_value else None
        self.maximum_value = options.maximum_value if options.maximum_value else None
        self.sum = options.sum if options.sum else None
        self.calculate_fixed_timing_hour = options.calculate_fixed_timing_hour if options.calculate_fixed_timing_hour else None
        self.calculate_fixed_timing_minute = options.calculate_fixed_timing_minute if options.calculate_fixed_timing_minute else None
        self.calculate_interval_minutes = options.calculate_interval_minutes if options.calculate_interval_minutes else None
        self.additional_scopes = options.additional_scopes if options.additional_scopes else None
        self.entry_period_event_id = options.entry_period_event_id if options.entry_period_event_id else None
        self.access_period_event_id = options.access_period_event_id if options.access_period_event_id else None
        self.ignore_user_ids = options.ignore_user_ids if options.ignore_user_ids else None
        self.generation = options.generation if options.generation else None

    @staticmethod
    def scope_is_global(
        name: str,
        order_direction: CategoryModelOrderDirection,
        unique_by_user_id: bool,
        calculate_interval_minutes: int,
        options: Optional[CategoryModelScopeIsGlobalOptions] = CategoryModelScopeIsGlobalOptions(),
    ) -> CategoryModel:
        return CategoryModel(
            name,
            order_direction,
            CategoryModelScope.GLOBAL,
            unique_by_user_id,
            CategoryModelOptions(
                calculate_interval_minutes,
                options.metadata,
                options.minimum_value,
                options.maximum_value,
                options.calculate_fixed_timing_hour,
                options.calculate_fixed_timing_minute,
                options.additional_scopes,
                options.entry_period_event_id,
                options.access_period_event_id,
                options.ignore_user_ids,
                options.generation,
            ),
        )

    @staticmethod
    def scope_is_scoped(
        name: str,
        order_direction: CategoryModelOrderDirection,
        unique_by_user_id: bool,
        options: Optional[CategoryModelScopeIsScopedOptions] = CategoryModelScopeIsScopedOptions(),
    ) -> CategoryModel:
        return CategoryModel(
            name,
            order_direction,
            CategoryModelScope.SCOPED,
            unique_by_user_id,
            CategoryModelOptions(
                options.metadata,
                options.minimum_value,
                options.maximum_value,
                options.calculate_fixed_timing_hour,
                options.calculate_fixed_timing_minute,
                options.additional_scopes,
                options.entry_period_event_id,
                options.access_period_event_id,
                options.ignore_user_ids,
                options.generation,
            ),
        )

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.minimum_value is not None:
            properties["minimumValue"] = self.minimum_value
        if self.maximum_value is not None:
            properties["maximumValue"] = self.maximum_value
        if self.order_direction is not None:
            properties["orderDirection"] = self.order_direction.value
        if self.scope is not None:
            properties["scope"] = self.scope.value
        if self.unique_by_user_id is not None:
            properties["uniqueByUserId"] = self.unique_by_user_id
        if self.sum is not None:
            properties["sum"] = self.sum
        if self.calculate_fixed_timing_hour is not None:
            properties["calculateFixedTimingHour"] = self.calculate_fixed_timing_hour
        if self.calculate_fixed_timing_minute is not None:
            properties["calculateFixedTimingMinute"] = self.calculate_fixed_timing_minute
        if self.calculate_interval_minutes is not None:
            properties["calculateIntervalMinutes"] = self.calculate_interval_minutes
        if self.additional_scopes is not None:
            properties["additionalScopes"] = [
                v.properties(
                )
                for v in self.additional_scopes
            ]
        if self.entry_period_event_id is not None:
            properties["entryPeriodEventId"] = self.entry_period_event_id
        if self.access_period_event_id is not None:
            properties["accessPeriodEventId"] = self.access_period_event_id
        if self.ignore_user_ids is not None:
            properties["ignoreUserIds"] = self.ignore_user_ids
        if self.generation is not None:
            properties["generation"] = self.generation

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .enum.NamespaceType import NamespaceType
from .options.NamespaceTypeIsGs2Options import NamespaceTypeIsGs2Options
from .options.NamespaceTypeIsBigqueryOptions import NamespaceTypeIsBigqueryOptions
from .options.NamespaceTypeIsFirehoseOptions import NamespaceTypeIsFirehoseOptions
from .AccessLog import AccessLog
from .options.AccessLogOptions import AccessLogOptions
from .AccessLogCount import AccessLogCount
from .options.AccessLogCountOptions import AccessLogCountOptions
from .IssueStampSheetLog import IssueStampSheetLog
from .options.IssueStampSheetLogOptions import IssueStampSheetLogOptions
from .IssueStampSheetLogCount import IssueStampSheetLogCount
from .options.IssueStampSheetLogCountOptions import IssueStampSheetLogCountOptions
from .ExecuteStampSheetLog import ExecuteStampSheetLog
from .options.ExecuteStampSheetLogOptions import ExecuteStampSheetLogOptions
from .ExecuteStampSheetLogCount import ExecuteStampSheetLogCount
from .options.ExecuteStampSheetLogCountOptions import ExecuteStampSheetLogCountOptions
from .ExecuteStampTaskLog import ExecuteStampTaskLog
from .options.ExecuteStampTaskLogOptions import ExecuteStampTaskLogOptions
from .ExecuteStampTaskLogCount import ExecuteStampTaskLogCount
from .options.ExecuteStampTaskLogCountOptions import ExecuteStampTaskLogCountOptions


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .AreaModel import AreaModel
from .options.AreaModelOptions import AreaModelOptions
from .LayerModel import LayerModel
from .options.LayerModelOptions import LayerModelOptions
from .Layer import Layer
from .options.LayerOptions import LayerOptions
from .Position import Position
from .options.PositionOptions import PositionOptions
from .MyPosition import MyPosition
from .options.MyPositionOptions import MyPositionOptions
from .Scope import Scope
from .options.ScopeOptions import ScopeOptions
from .Vector import Vector
from .options.VectorOptions import VectorOptions
from .CurrentMasterData import CurrentMasterData


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .FormModel import FormModel
from .options.FormModelOptions import FormModelOptions
from .MoldModel import MoldModel
from .options.MoldModelOptions import MoldModelOptions
from .PropertyFormModel import PropertyFormModel
from .options.PropertyFormModelOptions import PropertyFormModelOptions
from .Slot import Slot
from .options.SlotOptions import SlotOptions
from .SlotModel import SlotModel
from .options.SlotModelOptions import SlotModelOptions
from .SlotWithSignature import SlotWithSignature
from .options.SlotWithSignatureOptions import SlotWithSignatureOptions
from .enum.SlotWithSignaturePropertyType import SlotWithSignaturePropertyType
from .AcquireActionConfig import AcquireActionConfig
from .options.AcquireActionConfigOptions import AcquireActionConfigOptions
from .CurrentMasterData import CurrentMasterData


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from ...core.model import ConsumeAction
from ...core.model import AcquireAction
from .options.IncrementalRateModelOptions import IncrementalRateModelOptions
from .options.IncrementalRateModelCalculateTypeIsLinearOptions import IncrementalRateModelCalculateTypeIsLinearOptions
from .options.IncrementalRateModelCalculateTypeIsPowerOptions import IncrementalRateModelCalculateTypeIsPowerOptions
from .options.IncrementalRateModelCalculateTypeIsGs2ScriptOptions import IncrementalRateModelCalculateTypeIsGs2ScriptOptions
from .enum.IncrementalRateModelCalculateType import IncrementalRateModelCalculateType


class IncrementalRateModel:
    name: str
    consume_action: ConsumeAction
    calculate_type: IncrementalRateModelCalculateType
    exchange_count_id: str
    maximum_exchange_count: int
    metadata: Optional[str] = None
    base_value: Optional[int] = None
    coefficient_value: Optional[int] = None
    calculate_script_id: Optional[str] = None
    acquire_actions: Optional[List[AcquireAction]] = None

    def __init__(
        self,
        name: str,
        consume_action: ConsumeAction,
        calculate_type: IncrementalRateModelCalculateType,
        exchange_count_id: str,
        maximum_exchange_count: int,
        options: Optional[IncrementalRateModelOptions] = IncrementalRateModelOptions(),
    ):
        self.name = name
        self.consume_action = consume_action
        self.calculate_type = calculate_type
        self.exchange_count_id = exchange_count_id
        self.maximum_exchange_count = maximum_exchange_count
        self.metadata = options.metadata if options.metadata else None
        self.base_value = options.base_value if options.base_value else None
        self.coefficient_value = options.coefficient_value if options.coefficient_value else None
        self.calculate_script_id = options.calculate_script_id if options.calculate_script_id else None
        self.acquire_actions = options.acquire_actions if options.acquire_actions else None

    @staticmethod
    def calculate_type_is_linear(
        name: str,
        consume_action: ConsumeAction,
        exchange_count_id: str,
        maximum_exchange_count: int,
        base_value: int,
        coefficient_value: int,
        options: Optional[IncrementalRateModelCalculateTypeIsLinearOptions] = IncrementalRateModelCalculateTypeIsLinearOptions(),
    ) -> IncrementalRateModel:
        return IncrementalRateModel(
            name,
            consume_action,
            IncrementalRateModelCalculateType.LINEAR,
            exchange_count_id,
            maximum_exchange_count,
            IncrementalRateModelOptions(
                base_value,
                coefficient_value,
                options.metadata,
                options.acquire_actions,
            ),
        )

    @staticmethod
    def calculate_type_is_power(
        name: str,
        consume_action: ConsumeAction,
        exchange_count_id: str,
        maximum_exchange_count: int,
        coefficient_value: int,
        options: Optional[IncrementalRateModelCalculateTypeIsPowerOptions] = IncrementalRateModelCalculateTypeIsPowerOptions(),
    ) -> IncrementalRateModel:
        return IncrementalRateModel(
            name,
            consume_action,
            IncrementalRateModelCalculateType.POWER,
            exchange_count_id,
            maximum_exchange_count,
            IncrementalRateModelOptions(
                coefficient_value,
                options.metadata,
                options.acquire_actions,
            ),
        )

    @staticmethod
    def calculate_type_is_gs2_script(
        name: str,
        consume_action: ConsumeAction,
        exchange_count_id: str,
        maximum_exchange_count: int,
        calculate_script_id: str,
        options: Optional[IncrementalRateModelCalculateTypeIsGs2ScriptOptions] = IncrementalRateModelCalculateTypeIsGs2ScriptOptions(),
    ) -> IncrementalRateModel:
        return IncrementalRateModel(
            name,
            consume_action,
            IncrementalRateModelCalculateType.GS2_SCRIPT,
            exchange_count_id,
            maximum_exchange_count,
            IncrementalRateModelOptions(
                calculate_script_id,
                options.metadata,
                options.acquire_actions,
            ),
        )

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.consume_action is not None:
            properties["consumeAction"] = self.consume_action.properties(
            )
        if self.calculate_type is not None:
            properties["calculateType"] = self.calculate_type.value
        if self.base_value is not None:
            properties["baseValue"] = self.base_value
        if self.coefficient_value is not None:
            properties["coefficientValue"] = self.coefficient_value
        if self.calculate_script_id is not None:
            properties["calculateScriptId"] = self.calculate_script_id
        if self.exchange_count_id is not None:
            properties["exchangeCountId"] = self.exchange_count_id
        if self.maximum_exchange_count is not None:
            properties["maximumExchangeCount"] = self.maximum_exchange_count
        if self.acquire_actions is not None:
            properties["acquireActions"] = [
                v.properties(
                )
                for v in self.acquire_actions
            ]

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.LayerOptions import LayerOptions


class Layer:
    area_model_name: str
    layer_model_name: str
    number_of_min_entries: int
    number_of_max_entries: int
    height: int
    root: Optional[str] = None

    def __init__(
        self,
        area_model_name: str,
        layer_model_name: str,
        number_of_min_entries: int,
        number_of_max_entries: int,
        height: int,
        options: Optional[LayerOptions] = LayerOptions(),
    ):
        self.area_model_name = area_model_name
        self.layer_model_name = layer_model_name
        self.number_of_min_entries = number_of_min_entries
        self.number_of_max_entries = number_of_max_entries
        self.height = height
        self.root = options.root if options.root else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.area_model_name is not None:
            properties["areaModelName"] = self.area_model_name
        if self.layer_model_name is not None:
            properties["layerModelName"] = self.layer_model_name
        if self.root is not None:
            properties["root"] = self.root
        if self.number_of_min_entries is not None:
            properties["numberOfMinEntries"] = self.number_of_min_entries
        if self.number_of_max_entries is not None:
            properties["numberOfMaxEntries"] = self.number_of_max_entries
        if self.height is not None:
            properties["height"] = self.height

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.func import GetAttr, Join
from .BalanceParameterModelRef import BalanceParameterModelRef
from .RarityParameterModelRef import RarityParameterModelRef
from ..stamp_sheet.ReDrawBalanceParameterStatusByUserId import ReDrawBalanceParameterStatusByUserId
from ..stamp_sheet.SetBalanceParameterStatusByUserId import SetBalanceParameterStatusByUserId
from ..stamp_sheet.ReDrawRarityParameterStatusByUserId import ReDrawRarityParameterStatusByUserId
from ..stamp_sheet.AddRarityParameterStatusByUserId import AddRarityParameterStatusByUserId
from ..stamp_sheet.SetRarityParameterStatusByUserId import SetRarityParameterStatusByUserId
from ..stamp_sheet.VerifyRarityParameterStatusByUserId import VerifyRarityParameterStatusByUserId


class NamespaceRef:
    namespace_name: str

    def __init__(
        self,
        namespace_name: str,
    ):
        self.namespace_name = namespace_name

    def balance_parameter_model(
        self,
        parameter_name: str,
    ) -> BalanceParameterModelRef:
        return BalanceParameterModelRef(
            self.namespace_name,
            parameter_name,
        )

    def rarity_parameter_model(
        self,
        parameter_name: str,
    ) -> RarityParameterModelRef:
        return RarityParameterModelRef(
            self.namespace_name,
            parameter_name,
        )

    def re_draw_balance_parameter_status(
        self,
        parameter_name: str,
        property_id: str,
        fixed_parameter_names: Optional[List[str]] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> ReDrawBalanceParameterStatusByUserId:
        return ReDrawBalanceParameterStatusByUserId(
            self.namespace_name,
            parameter_name,
            property_id,
            fixed_parameter_names,
            user_id,
        )

    def set_balance_parameter_status(
        self,
        parameter_name: str,
        property_id: str,
        parameter_values: List[BalanceParameterValue],
        user_id: Optional[str] = "#{userId}",
    ) -> SetBalanceParameterStatusByUserId:
        return SetBalanceParameterStatusByUserId(
            self.namespace_name,
            parameter_name,
            property_id,
            parameter_values,
            user_id,
        )

    def re_draw_rarity_parameter_status(
        self,
        parameter_name: str,
        property_id: str,
        fixed_parameter_names: Optional[List[str]] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> ReDrawRarityParameterStatusByUserId:
        return ReDrawRarityParameterStatusByUserId(
            self.namespace_name,
            parameter_name,
            property_id,
            fixed_parameter_names,
            user_id,
        )

    def add_rarity_parameter_status(
        self,
        parameter_name: str,
        property_id: str,
        count: Optional[int] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> AddRarityParameterStatusByUserId:
        return AddRarityParameterStatusByUserId(
            self.namespace_name,
            parameter_name,
            property_id,
            count,
            user_id,
        )

    def set_rarity_parameter_status(
        self,
        parameter_name: str,
        property_id: str,
        parameter_values: Optional[List[RarityParameterValue]] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> SetRarityParameterStatusByUserId:
        return SetRarityParameterStatusByUserId(
            self.namespace_name,
            parameter_name,
            property_id,
            parameter_values,
            user_id,
        )

    def verify_rarity_parameter_status(
        self,
        parameter_name: str,
        property_id: str,
        verify_type: str,
        parameter_value_name: Optional[str] = None,
        parameter_count: Optional[int] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> VerifyRarityParameterStatusByUserId:
        return VerifyRarityParameterStatusByUserId(
            self.namespace_name,
            parameter_name,
            property_id,
            verify_type,
            parameter_value_name,
            parameter_count,
            user_id,
        )

    def grn(
        self,
    ) -> str:
        return Join(
            ":",
            [
                "grn",
                "gs2",
                GetAttr.region(
                ).str(
                ),
                GetAttr.owner_id(
                ).str(
                ),
                "enchant",
                self.namespace_name,
            ],
        ).str(
        )
